//! Peer: someone we can communicate with directly

use std::{rc::Rc, collections::HashMap, cell::RefCell};

use crate::prelude::*;

use super::msg::Message;


#[derive(Clone)]
pub struct Peer {
    inner: Rc<dyn InnerPeer>
}

impl std::fmt::Debug for Peer {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        f.write_str("Peer")?;
        Ok(())
    }
}

trait InnerPeer {
    fn dispatch(&self, api: &crate::Fleck, msg: super::msg::Message);
}

struct UdpPeer<SourceTag: 'static, SinkTag: 'static> {
    _ghost: std::marker::PhantomData<(SourceTag, SinkTag)>,
}

impl<SourceTag: 'static, SinkTag: 'static> UdpPeer<SourceTag, SinkTag> {
    fn forward(&self, api: crate::Fleck, msg: (std::net::SocketAddr, Vec<u8>)) -> Option<(std::net::SocketAddr, Vec<u8>)> {
        None
    }
}

impl<SourceTag: 'static, SinkTag: 'static> InnerPeer for UdpPeer<SourceTag, SinkTag> {
    fn dispatch(&self, api: &crate::Fleck, msg: fleck_core::msg::Message) {
        
        api.queue::<(SinkTag, (std::net::SocketAddr, Vec<u8>))>(todo!());
    }
}

struct UdpSocket {
    peers: HashMap<std::net::SocketAddr, Rc<Peer>>,
}

impl UdpSocket {
    fn handle_incoming(&self, ) {
        
    }

    fn handle_outgoing(&self, ) {
        
    }
}

#[derive(Default)]
pub struct PeerService {
    udp: RefCell<Vec<Rc<UdpSocket>>>,
}

impl Service for std::rc::Rc<PeerService> {
    fn register_channels(&self, eroot: &mut EventRoot) {
        eroot.channel::<fleck_core::SendPacketChannel>().sub_opt(fleck_core::SendOrder::Send, self, PeerService::dispatch);
    }
}

impl PeerService {
    fn dispatch(&self, api: &crate::Fleck, msg: super::msg::Message) -> Option<super::msg::Message> {
        /*let peer = match &msg.peer {
            Some(peer) => peer.clone(),
            None => {
                log::error!("Packet got to peer dispatch with no peer?");
                return None
            }
        };

        peer.inner.dispatch(api, msg);*/

        None
    }

    pub fn link_udp_socket<SourceTag: 'static, SinkTag: 'static>(&self) {
        self.udp.borrow_mut().push(
            Rc::new(UdpSocket {
                peers: Default::default()
            })
        );
    }
}
